/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param;

import java.util.Collection;

import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.param.lib.MethodenBibliothek;
import de.bsvrz.puk.param.lib.Parameter;
import de.bsvrz.puk.param.lib.ParameterClientException;
import de.bsvrz.puk.param.lib.ParameterInfo;
import de.bsvrz.puk.param.lib.ParameterManager;
import de.bsvrz.puk.param.lib.QuellTyp;
import de.bsvrz.puk.param.lib.daten.DataWithTime;

/**
 * Prüft die Funtionalität der Parametrierung entsprechend der vorgegebenen
 * Testspezifikation.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
@Ignore("Test-DAV-Verbindung prüfen")
public class TestSpezifikation implements ClientSenderInterface {

	/**
	 * Name des Attributs zur Speicherung einer Datenspezifikation.
	 */
	private static final String DATEN_SPEZIFIKATION = "DatenSpezifikation";

	/**
	 * Name des Attributs zur Speicherung eines Parametersatzes.
	 */
	private static final String PARAMETER_SATZ = "ParameterSatz";

	/**
	 * Inhalt des Parameters, wenn der Parameter am 3sp MQ gesetzt wird.
	 */
	private static final String GLOBAL_MQ = "GLOBAL MQ";

	/**
	 * Inhalt des Parameters, wenn der Parameter am MQ2 gesetzt wird.
	 */
	private static final String GLOBAL_MQ2 = "GLOBAL MQ2";

	/**
	 * Inhalt des Parameters, wenn der Parameter am MQ2 verändert wird wird.
	 */
	private static final String GLOBAL_MQ2_NEU = "GLOBAL MQ2 Neu";

	/**
	 * globale Instanz der Klasse zur Anmeldung als Sender.
	 */
	private static TestSpezifikation instance = new TestSpezifikation();

	/**
	 * die verwendete Datenverteilerverbindung.
	 */
	private static ClientDavConnection connection;

	/**
	 * die Attributgruppe für die Parametrierung der Parametrierung.
	 */
	private static AttributeGroup parametrierungsAtg;

	/**
	 * das Objekt, welches die Parametrierung repräsentiert.
	 */
	private static SystemObject kv;

	/**
	 * der verwendete 3sp MQ.
	 */
	private static SystemObject mq;

	/**
	 * der verwendete MQ1.
	 */
	private static SystemObject mq1;

	/**
	 * der verwendete MQ2.
	 */
	private static SystemObject mq2;

	/**
	 * der verwendete Fahrstreifen 1.
	 */
	private static SystemObject fs1;

	/**
	 * der verwendete Fahrstreifen 2.
	 */
	private static SystemObject fs2;

	/**
	 * der verwendete Fahrstreifen 3.
	 */
	private static SystemObject fs3;

	/**
	 * der verwendete Fahrstreifen 4.
	 */
	private static SystemObject fs4;

	/**
	 * der verwendete Fahrstreifen 5.
	 */
	private static SystemObject fs5;

	/**
	 * der verwendete Fahrstreifen 6.
	 */
	private static SystemObject fs6;

	/**
	 * die Attributgrupppe für die Fahrstreifenparameter.
	 */
	private static AttributeGroup fsParamAtg;

	/**
	 * der Aspekt für die Vorgabe von Parametern.
	 */
	private static Aspect paramVorgabeAspekt;

	/**
	 * stellt die Verbindung zum Testdatenverteiler auf localhost:8083 her.
	 */
	@BeforeClass
	public static void datenverteilerVerbindung() {
		try {
			TestSpezifikation.connection = new ClientDavConnection(new ClientDavParameters());
			TestSpezifikation.connection.connect();
			TestSpezifikation.connection.login("Tester", "geheim");

			TestSpezifikation.kv = TestSpezifikation.connection.getLocalConfigurationAuthority();
			TestSpezifikation.parametrierungsAtg = TestSpezifikation.connection.getDataModel()
					.getAttributeGroup("atg.parametrierung");
			TestSpezifikation.mq = TestSpezifikation.connection.getDataModel().getObject("paramtest.mq");
			TestSpezifikation.mq1 = TestSpezifikation.connection.getDataModel().getObject("paramtest.mq1");
			TestSpezifikation.mq2 = TestSpezifikation.connection.getDataModel().getObject("paramtest.mq2");
			TestSpezifikation.fs1 = TestSpezifikation.connection.getDataModel().getObject("paramtest.fs1");
			TestSpezifikation.fs2 = TestSpezifikation.connection.getDataModel().getObject("paramtest.fs2");
			TestSpezifikation.fs3 = TestSpezifikation.connection.getDataModel().getObject("paramtest.fs3");
			TestSpezifikation.fs4 = TestSpezifikation.connection.getDataModel().getObject("paramtest.fs4");
			TestSpezifikation.fs5 = TestSpezifikation.connection.getDataModel().getObject("paramtest.fs5");
			TestSpezifikation.fs6 = TestSpezifikation.connection.getDataModel().getObject("paramtest.fs6");
			TestSpezifikation.fsParamAtg = TestSpezifikation.connection.getDataModel()
					.getAttributeGroup("paramtest.atg.fs");

			TestSpezifikation.paramVorgabeAspekt = TestSpezifikation.connection.getDataModel()
					.getAspect("asp.parameterVorgabe");

			TestSpezifikation.connection.subscribeSender(TestSpezifikation.instance, TestSpezifikation.kv,
					new DataDescription(TestSpezifikation.parametrierungsAtg, TestSpezifikation.paramVorgabeAspekt),
					SenderRole.sender());

		} catch (final MissingParameterException e) {
			Assert.fail(e.getMessage());
		} catch (final CommunicationError e) {
			Assert.fail(e.getMessage());
		} catch (final ConnectionException e) {
			Assert.fail(e.getMessage());
		} catch (final InconsistentLoginException e) {
			Assert.fail(e.getMessage());
		} catch (final OneSubscriptionPerSendData e) {
			Assert.fail(e.getMessage());
		}
	}

	/**
	 * beendet dei Datenverteilerverbindung nach Abschluss der Tests.
	 */
	@AfterClass
	public static void datenverteilerVerbindungBeenden() {
		if (TestSpezifikation.connection != null) {
			TestSpezifikation.connection.disconnect(false, "");
		}
	}

	/**
	 * führt einen Durchgang zum setzen und Auslesen von Parametern aus.
	 *
	 * @param leer
	 *            die Parameter für Fahrstreifen sind parametriert und können
	 *            abgefragt werden.
	 */
	private void ablauf(final boolean leer) {
		final ParameterManager mgr = MethodenBibliothek.getParameterManager(TestSpezifikation.connection);

		TestSpezifikation.connection.sleep(5000);

		try {

			System.err.println("Bereinige Globale Parameter");
			mgr.setParameter(new Parameter(new ParameterInfo(TestSpezifikation.mq, TestSpezifikation.fs1.getType(),
					TestSpezifikation.fsParamAtg, (short) 0)));
			mgr.setParameter(new Parameter(new ParameterInfo(TestSpezifikation.mq2, TestSpezifikation.fs1.getType(),
					TestSpezifikation.fsParamAtg, (short) 0)));

			System.err.println(
					"Setzte globale Parameter für Fahrstreifen an 3spMQ auf \"" + TestSpezifikation.GLOBAL_MQ + "\"");
			mgr.setParameter(new Parameter(new ParameterInfo(TestSpezifikation.mq, TestSpezifikation.fs1.getType(),
					TestSpezifikation.fsParamAtg, (short) 0), datenSatz(TestSpezifikation.GLOBAL_MQ)));

			System.err.println(
					"Setzte globale Parameter für Fahrstreifen an MQ2 auf \"" + TestSpezifikation.GLOBAL_MQ2 + "\"");
			mgr.setParameter(new Parameter(new ParameterInfo(TestSpezifikation.mq2, TestSpezifikation.fs1.getType(),
					TestSpezifikation.fsParamAtg, (short) 0), datenSatz(TestSpezifikation.GLOBAL_MQ2)));

			Parameter parameter = mgr
					.getParameter(new ParameterInfo(TestSpezifikation.fs3, TestSpezifikation.fsParamAtg))[0];
			pruefeParameter(parameter, TestSpezifikation.fs3, QuellTyp.LOKAL_HIERARCHIE, TestSpezifikation.mq,
					TestSpezifikation.GLOBAL_MQ, leer);
			parameter = mgr.getParameter(new ParameterInfo(TestSpezifikation.fs4, TestSpezifikation.fsParamAtg))[0];
			pruefeParameter(parameter, TestSpezifikation.fs4, QuellTyp.LOKAL_HIERARCHIE, TestSpezifikation.mq2,
					TestSpezifikation.GLOBAL_MQ2, leer);

			System.err.println("Setzte globale Parameter für Fahrstreifen an MQ2 auf \""
					+ TestSpezifikation.GLOBAL_MQ2_NEU + "\"");
			mgr.setParameter(new Parameter(new ParameterInfo(TestSpezifikation.mq2, TestSpezifikation.fs1.getType(),
					TestSpezifikation.fsParamAtg, (short) 0), datenSatz(TestSpezifikation.GLOBAL_MQ2_NEU)));

			parameter = mgr.getParameter(new ParameterInfo(TestSpezifikation.fs3, TestSpezifikation.fsParamAtg))[0];
			pruefeParameter(parameter, TestSpezifikation.fs3, QuellTyp.LOKAL_HIERARCHIE, TestSpezifikation.mq,
					TestSpezifikation.GLOBAL_MQ, leer);
			parameter = mgr.getParameter(new ParameterInfo(TestSpezifikation.fs4, TestSpezifikation.fsParamAtg))[0];
			pruefeParameter(parameter, TestSpezifikation.fs4, QuellTyp.LOKAL_HIERARCHIE, TestSpezifikation.mq2,
					TestSpezifikation.GLOBAL_MQ2_NEU, leer);

			System.err.println("Lösche globale Parameter für Fahrstreifen an MQ2");
			mgr.setParameter(new Parameter(new ParameterInfo(TestSpezifikation.mq2, TestSpezifikation.fs1.getType(),
					TestSpezifikation.fsParamAtg, (short) 0)));

			parameter = mgr.getParameter(new ParameterInfo(TestSpezifikation.fs3, TestSpezifikation.fsParamAtg))[0];
			pruefeParameter(parameter, TestSpezifikation.fs3, QuellTyp.LOKAL_HIERARCHIE, TestSpezifikation.mq,
					TestSpezifikation.GLOBAL_MQ, leer);
			parameter = mgr.getParameter(new ParameterInfo(TestSpezifikation.fs4, TestSpezifikation.fsParamAtg))[0];
			pruefeParameter(parameter, TestSpezifikation.fs4, QuellTyp.LOKAL_HIERARCHIE, TestSpezifikation.mq,
					TestSpezifikation.GLOBAL_MQ, leer);

			System.err.println("Ermittle änderbare Objekte vom Typ "
					+ TestSpezifikation.fs1.getType().getNameOrPidOrId() + " am 3sp MQ");
			final Collection<SystemObject> objekte = mgr.getAenderbareObjekte(new ParameterInfo(TestSpezifikation.mq,
					TestSpezifikation.fs1.getType(), TestSpezifikation.fsParamAtg));
			final SystemObject[] erwartetetObjekte = { TestSpezifikation.fs1, TestSpezifikation.fs2,
					TestSpezifikation.fs3, TestSpezifikation.fs4, TestSpezifikation.fs5, TestSpezifikation.fs6 };
			for (final SystemObject obj : erwartetetObjekte) {
				if (!objekte.contains(obj)) {
					Assert.fail("Erwartetes Objekt: " + obj.getNameOrPidOrId()
							+ " ist nicht in der Menge der ermittelten Objekte entahlten");
				}
				objekte.remove(obj);
			}
			if (objekte.size() > 0) {
				Assert.fail("Unerwartete Objekte erhalten");
			}
			System.err.println("Anderbare Objekte entsprechen den erwarteten");

			System.err.println("Ermittle Parameterwurzeln für Fahrstreifen");

			final SystemObject[] wurzeln = mgr.getParameterWurzel(
					new ParameterInfo[] { new ParameterInfo(TestSpezifikation.fs1, TestSpezifikation.fsParamAtg),
							new ParameterInfo(TestSpezifikation.fs2, TestSpezifikation.fsParamAtg),
							new ParameterInfo(TestSpezifikation.fs3, TestSpezifikation.fsParamAtg),
							new ParameterInfo(TestSpezifikation.fs4, TestSpezifikation.fsParamAtg),
							new ParameterInfo(TestSpezifikation.fs5, TestSpezifikation.fsParamAtg),
							new ParameterInfo(TestSpezifikation.fs6, TestSpezifikation.fsParamAtg) });

			Assert.assertEquals(6, wurzeln.length);
			for (final SystemObject obj : wurzeln) {
				if (leer) {
					Assert.assertEquals(null, obj);
				} else {
					Assert.assertEquals(TestSpezifikation.mq, obj);
				}
			}

			System.err.println("Parameterwurzeln für Fahrstreifen sind wie erwartet");

		} catch (final ParameterClientException e) {
			Assert.fail(e.getMessage());
		}
	}

	@Override
	public void dataRequest(final SystemObject object, final DataDescription dataDescription, final byte state) {
		// die Funktion sollte nicht aufgerufen werden, weil Sendesteuerung
		// nicht unterstützt wird.
	}

	/**
	 * füllt den Datensatz zur Parametrierung eines Fahrstreifens aus.
	 *
	 * @param string
	 *            der Text, der in den Parameter eingetragen werden soll.
	 * @return der gefüllte Datensatz
	 */
	private DataWithTime datenSatz(final String string) {
		final Data daten = TestSpezifikation.connection.createData(TestSpezifikation.fsParamAtg);
		daten.getTextValue("Parameter").setText(string);
		return new DataWithTime(daten, System.currentTimeMillis());
	}

	/**
	 * führt den ersten Testdurchlauf aus.
	 */
	@Test
	public void durchlauf1() {
		System.err.println("============================================================================");
		System.err.println("Durchlauf 1");
		System.err.println("Parameterdatensatz enthält einen Bereich und die Referenzen auf Fahrstreifen");
		System.err.println("============================================================================");
		parametrierungsDatensatzMitBereichUndMitPidsUndAtgWildcard();
		ablauf(false);
	}

	/**
	 * führt den zweiten Testdurchlauf aus.
	 */
	@Test
	public void durchlauf2() {
		System.err.println("============================================================================");
		System.err.println("Durchlauf 2");
		System.err.println("Parameterdatensatz enthält nur einen Bereich");
		System.err.println("============================================================================");
		parametrierungsDatensatzMitBereichUndMitObjWildcardUndAtgWildcard();
		ablauf(false);
	}

	/**
	 * führt den dritten Testdurchlauf aus.
	 */
	@Test
	public void durchlauf3() {
		System.err.println("============================================================================");
		System.err.println("Durchlauf 3");
		System.err.println("Parameterdatensatz enthält nur Wildcards");
		System.err.println("============================================================================");
		parametrierungsDatensatzMitObjWildcardUndAtgWildcard();
		ablauf(false);
	}

	/**
	 * führt den vierten Testdurchlauf aus.
	 */
	@Test
	public void durchlauf4() {
		System.err.println("============================================================================");
		System.err.println("Durchlauf 4");
		System.err.println("Parameterdatensatz enthält Objekt als Menge");
		System.err.println("============================================================================");
		parametrierungsDatensatzMitObjectePerMengeUndAtgWildcard();
		ablauf(false);
	}

	/**
	 * führt den fünften Testdurchlauf aus.
	 */
	@Test
	public void durchlauf5() {
		System.err.println("============================================================================");
		System.err.println("Durchlauf 5");
		System.err.println("Parameterdatensatz enthält Objekt als Typ");
		System.err.println("============================================================================");
		parametrierungsDatensatzMitObjectsPerTypUndAtgWildcard();
		ablauf(false);
	}

	/**
	 * führt den sechsten Testdurchlauf aus.
	 */
	@Test
	public void durchlauf6() {
		System.err.println("============================================================================");
		System.err.println("Durchlauf 6");
		System.err.println("Parameterdatensatz ist leer");
		System.err.println("============================================================================");
		parametrierungsDatensatzLeer();
		ablauf(true);
	}

	@Override
	public boolean isRequestSupported(final SystemObject object, final DataDescription dataDescription) {
		return false;
	}

	/**
	 * leert den Parametrierungsdatensatz.
	 */
	private void parametrierungsDatensatzLeer() {
		final Data daten = TestSpezifikation.connection.createData(TestSpezifikation.parametrierungsAtg);

		daten.getArray(TestSpezifikation.PARAMETER_SATZ).setLength(0);

		try {
			TestSpezifikation.connection.sendData(new ResultData(TestSpezifikation.kv,
					new DataDescription(TestSpezifikation.parametrierungsAtg, TestSpezifikation.paramVorgabeAspekt),
					System.currentTimeMillis(), daten));
		} catch (final SendSubscriptionNotConfirmed e) {
			Assert.fail(e.getMessage());
		}
	}

	/**
	 * Parametriert die Parametrierung für einen Bereich.
	 */
	private void parametrierungsDatensatzMitBereichUndMitObjWildcardUndAtgWildcard() {
		final Data daten = TestSpezifikation.connection.createData(TestSpezifikation.parametrierungsAtg);

		daten.getArray(TestSpezifikation.PARAMETER_SATZ).setLength(1);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray("Bereich").setLength(1);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray("Bereich").getReferenceValue(0)
				.setSystemObject(TestSpezifikation.fs1.getConfigurationArea());
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.setLength(1);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("Objekt").setLength(0);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("AttributGruppe").setLength(0);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getUnscaledValue("SimulationsVariante").set(0);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getItem("Einstellungen")
				.getUnscaledValue("Parametrieren").set(1);

		try {
			TestSpezifikation.connection.sendData(new ResultData(TestSpezifikation.kv,
					new DataDescription(TestSpezifikation.parametrierungsAtg, TestSpezifikation.paramVorgabeAspekt),
					System.currentTimeMillis(), daten));
		} catch (final SendSubscriptionNotConfirmed e) {
			Assert.fail(e.getMessage());
		}
	}

	/**
	 * Parametriert die Parametrierung für einen Bereich und eine Menge von
	 * Objekten.
	 */
	private void parametrierungsDatensatzMitBereichUndMitPidsUndAtgWildcard() {
		final Data daten = TestSpezifikation.connection.createData(TestSpezifikation.parametrierungsAtg);

		daten.getArray(TestSpezifikation.PARAMETER_SATZ).setLength(1);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray("Bereich").setLength(1);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray("Bereich").getReferenceValue(0)
				.setSystemObject(TestSpezifikation.fs1.getConfigurationArea());
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.setLength(1);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("Objekt").setLength(6);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("Objekt").getReferenceValue(0).setSystemObject(TestSpezifikation.fs1);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("Objekt").getReferenceValue(1).setSystemObject(TestSpezifikation.fs2);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("Objekt").getReferenceValue(2).setSystemObject(TestSpezifikation.fs3);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("Objekt").getReferenceValue(3).setSystemObject(TestSpezifikation.fs4);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("Objekt").getReferenceValue(4).setSystemObject(TestSpezifikation.fs5);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("Objekt").getReferenceValue(5).setSystemObject(TestSpezifikation.fs6);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("AttributGruppe").setLength(0);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getUnscaledValue("SimulationsVariante").set(0);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getItem("Einstellungen")
				.getUnscaledValue("Parametrieren").set(1);

		try {
			TestSpezifikation.connection.sendData(new ResultData(TestSpezifikation.kv,
					new DataDescription(TestSpezifikation.parametrierungsAtg, TestSpezifikation.paramVorgabeAspekt),
					System.currentTimeMillis(), daten));
		} catch (final SendSubscriptionNotConfirmed e) {
			Assert.fail(e.getMessage());
		}
	}

	/**
	 * parametriert die Parametrierung mit Mengen von Objekten.
	 */
	private void parametrierungsDatensatzMitObjectePerMengeUndAtgWildcard() {
		final Data daten = TestSpezifikation.connection.createData(TestSpezifikation.parametrierungsAtg);

		daten.getArray(TestSpezifikation.PARAMETER_SATZ).setLength(1);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray("Bereich").setLength(0);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.setLength(1);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("Objekt").setLength(2);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("Objekt").getReferenceValue(0)
				.setSystemObject(((ConfigurationObject) TestSpezifikation.mq1).getObjectSet("Fahrstreifen"));
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("Objekt").getReferenceValue(1)
				.setSystemObject(((ConfigurationObject) TestSpezifikation.mq2).getObjectSet("Fahrstreifen"));
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("AttributGruppe").setLength(0);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getUnscaledValue("SimulationsVariante").set(0);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getItem("Einstellungen")
				.getUnscaledValue("Parametrieren").set(1);

		try {
			TestSpezifikation.connection.sendData(new ResultData(TestSpezifikation.kv,
					new DataDescription(TestSpezifikation.parametrierungsAtg, TestSpezifikation.paramVorgabeAspekt),
					System.currentTimeMillis(), daten));
		} catch (final SendSubscriptionNotConfirmed e) {
			Assert.fail(e.getMessage());
		}
	}

	/**
	 * parametriert die Parametrierung mit dem Objekttyp.
	 */
	private void parametrierungsDatensatzMitObjectsPerTypUndAtgWildcard() {
		final Data daten = TestSpezifikation.connection.createData(TestSpezifikation.parametrierungsAtg);

		daten.getArray(TestSpezifikation.PARAMETER_SATZ).setLength(1);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray("Bereich").setLength(0);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.setLength(1);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("Objekt").setLength(1);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("Objekt").getReferenceValue(0).setSystemObject(TestSpezifikation.fs1.getType());
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("AttributGruppe").setLength(0);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getUnscaledValue("SimulationsVariante").set(0);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getItem("Einstellungen")
				.getUnscaledValue("Parametrieren").set(1);

		try {
			TestSpezifikation.connection.sendData(new ResultData(TestSpezifikation.kv,
					new DataDescription(TestSpezifikation.parametrierungsAtg, TestSpezifikation.paramVorgabeAspekt),
					System.currentTimeMillis(), daten));
		} catch (final SendSubscriptionNotConfirmed e) {
			Assert.fail(e.getMessage());
		}
	}

	/**
	 * parametriert die Parametrierung ausschliesslich mit Wildcards.
	 */
	private void parametrierungsDatensatzMitObjWildcardUndAtgWildcard() {
		final Data daten = TestSpezifikation.connection.createData(TestSpezifikation.parametrierungsAtg);

		daten.getArray(TestSpezifikation.PARAMETER_SATZ).setLength(1);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray("Bereich").setLength(0);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.setLength(1);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("Objekt").setLength(0);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getArray("AttributGruppe").setLength(0);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getArray(TestSpezifikation.DATEN_SPEZIFIKATION)
				.getItem(0).getUnscaledValue("SimulationsVariante").set(0);
		daten.getArray(TestSpezifikation.PARAMETER_SATZ).getItem(0).getItem("Einstellungen")
				.getUnscaledValue("Parametrieren").set(1);

		try {
			TestSpezifikation.connection.sendData(new ResultData(TestSpezifikation.kv,
					new DataDescription(TestSpezifikation.parametrierungsAtg, TestSpezifikation.paramVorgabeAspekt),
					System.currentTimeMillis(), daten));
		} catch (final SendSubscriptionNotConfirmed e) {
			Assert.fail(e.getMessage());
		}
	}

	/**
	 * prüft, ob der empfangene Parameterdatensatz den Erwartungen entspricht.
	 *
	 * @param parameter
	 *            der Parameter
	 * @param objekt
	 *            das erwartetete Objekt
	 * @param typ
	 *            der erwartete Typ der Quelle
	 * @param quellObj
	 *            die erwartete Quelle
	 * @param text
	 *            der erwartete Wert innerhalb des Datensatzes
	 * @param leer
	 *            Parameter enthält Daten, da das Objekt zum Parametrieren
	 *            freigegeben wurde
	 */
	private void pruefeParameter(final Parameter parameter, final SystemObject objekt, final QuellTyp typ,
			final SystemObject quellObj, final String text, final boolean leer) {
		Assert.assertEquals(parameter.getObjekt(), objekt);
		Assert.assertEquals(parameter.getAtg(), TestSpezifikation.fsParamAtg);
		if (leer) {
			Assert.assertEquals(null, parameter.getData());
		} else {
			Assert.assertEquals(parameter.getQuelle().getTyp(), typ);
			Assert.assertEquals(parameter.getQuelle().getObjekt(), quellObj);
			Assert.assertEquals(text, parameter.getData().getTextValue("Parameter").getText());
		}
		System.err.println("Parameter für " + objekt.getNameOrPidOrId()
				+ "entsprechen dem erwarteten Inhalt und der vorgesehenen Datenquelle");
	}
}
