/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.anfrage;

import java.io.Serializable;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.param.lib.ParametrierungsAnfrageTyp;
import de.bsvrz.puk.param.param.Parametrierung;

/**
 * Klasse zur Bearbeitung von Anfragen zur Steuerung der Parametrierung. Als Typ
 * wird der Antworttyp PARAMETER verwendet. Die Art der Steuerung wird über den
 * Index der Anfrage bestimmt. Momentan ist nur der Wert -1 für das Erstellen
 * eines Backups vorgesehen.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ControlAnfrageBeantworter extends ParameterAnfrageBeantworter {

	/**
	 * Konstruktor, erstellt einen Beantworter zur Steuerung der Parametrierung.
	 *
	 * @param parametrierung
	 *            die Parametrierung
	 * @param anfrager
	 *            das Objekt, das den Anfrage identifiziert
	 * @param index
	 *            der Index der Anfrage
	 */
	protected ControlAnfrageBeantworter(final Parametrierung parametrierung, final SystemObject anfrager,
			final int index) {
		super(parametrierung, anfrager, index);
	}

	@Override
	protected Serializable ermittleAntwort() {
		switch (getIndex()) {
		case -1:
			getParametrierung().getPersistenz().backup();
			break;
		default:
			break;
		}
		return Boolean.TRUE;
	}

	@Override
	protected ParametrierungsAnfrageTyp getAntwortTyp() {
		return ParametrierungsAnfrageTyp.PARAMETER;
	}
}
