/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.hierarchie;

import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Repräsentation eines Objekts in der Typhierarchie der Parametrierung.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
class TypHierarchieObjekt {

	/**
	 * der Name der Menge, die die untergeordneten Elemente des Objekts enthält.
	 */
	private final String menge;

	/** der Typ, den die untergeordneten Elemente haben. */
	private final SystemObjectType nachfolgerTyp;

	/** der Typ des Hierarchieobjekts. */
	private final SystemObjectType typ;

	/**
	 * Konstruktor, erzeugt ein Typhierarchie-Objekt für den übergebenen Typ,
	 * den Mengennamen und den Nachfolgertyp.
	 *
	 * @param typ
	 *            der Typ des Objekts
	 * @param menge
	 *            der Name der Menge, der die untergeordneten Objekte enthält
	 * @param nachfolger
	 *            der Typ der Nachfolgerelemente
	 */
	TypHierarchieObjekt(final SystemObjectType typ, final String menge, final SystemObjectType nachfolger) {
		if (typ == null) {
			throw new NullPointerException("Parameter \"typ\" darf nicht <null> sein");
		}
		this.typ = typ;
		this.menge = menge;
		this.nachfolgerTyp = nachfolger;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + (menge == null ? 0 : menge.hashCode());
		result = prime * result + (nachfolgerTyp == null ? 0 : nachfolgerTyp.hashCode());
		result = prime * result + (typ == null ? 0 : typ.hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (getClass() != obj.getClass()) {
			return false;
		}
		final TypHierarchieObjekt other = (TypHierarchieObjekt) obj;
		if (menge == null) {
			if (other.menge != null) {
				return false;
			}
		} else if (!menge.equals(other.menge)) {
			return false;
		}
		if (nachfolgerTyp == null) {
			if (other.nachfolgerTyp != null) {
				return false;
			}
		} else if (!nachfolgerTyp.equals(other.nachfolgerTyp)) {
			return false;
		}
		if (typ == null) {
			if (other.typ != null) {
				return false;
			}
		} else if (!typ.equals(other.typ)) {
			return false;
		}
		return true;
	}

	/**
	 * liefert den Namen der Menge, die die untergeordneten Objekte enthalten
	 * soll.
	 *
	 * @return den Name
	 */
	String getMenge() {
		return menge;
	}

	/**
	 * liefert den Typ der Nachfolgeobjekte.
	 *
	 * @return den Typ
	 */
	SystemObjectType getNachfolgerTyp() {
		return nachfolgerTyp;
	}

	/**
	 * liefert den Typ des Hierarchieobjekts.
	 *
	 * @return den Typ
	 */
	SystemObjectType getTyp() {
		return typ;
	}
}
