/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.lib.daten;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

/**
 * Klasse zum Test der Klasse Zeichenkette.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ZeichenketteTest {

	/** Regel zur Auswertung erwarteter Exceptions. */
	@Rule
	public ExpectedException thrown = ExpectedException.none();

	/**
	 * Test: Serialisierung und Deserialisierung.
	 */
	@Test
	public void testSerialization() {
		Zeichenkette zk = new Zeichenkette("ABC", "Willi");
		Assert.assertEquals("ABC", zk.getName());

		try {
			final ByteArrayOutputStream byteOut = new ByteArrayOutputStream(1000);
			final ObjectOutputStream output = new ObjectOutputStream(byteOut);
			output.writeObject(zk);
			final byte[] bytes = byteOut.toByteArray();

			final ObjectInputStream input = new ObjectInputStream(new ByteArrayInputStream(bytes));
			zk = (Zeichenkette) input.readObject();
			Assert.assertEquals("Willi", zk.get());
		} catch (final ClassNotFoundException | IOException e) {
			Assert.fail(e.getLocalizedMessage());
		}

	}

	/**
	 * Test: Serialisierung und Deserialisierung mit dem Wert <code>null</code>.
	 */
	@Test
	public void testSerializationNull() {
		Zeichenkette zk = new Zeichenkette("ABC", null);
		Assert.assertEquals("ABC", zk.getName());

		try {
			final ByteArrayOutputStream byteOut = new ByteArrayOutputStream(1000);
			final ObjectOutputStream output = new ObjectOutputStream(byteOut);
			output.writeObject(zk);
			final byte[] bytes = byteOut.toByteArray();

			final ObjectInputStream input = new ObjectInputStream(new ByteArrayInputStream(bytes));
			zk = (Zeichenkette) input.readObject();
			Assert.assertEquals(null, zk.get());
		} catch (final ClassNotFoundException | IOException e) {
			Assert.fail(e.getLocalizedMessage());
		}
	}

	/**
	 * Test: Wert setzen.
	 */
	@Test
	public void testSet() {
		final Zeichenkette zk = new Zeichenkette("Heinz");
		zk.set("Willi");
		Assert.assertEquals("Willi", zk.get());
		zk.set(null);
		Assert.assertEquals(null, zk.get());
	}

	/**
	 * Test: Konstruktor mit ATtributname.
	 */
	@SuppressWarnings("unused")
	@Test
	public void testZeichenketteString() {
		Assert.assertEquals(null, new Zeichenkette("Heinz").get());
		thrown.expect(IllegalArgumentException.class);
		new Zeichenkette(null);
	}

	/**
	 * Test: Konstruktor mit Attributname und Wert.
	 */
	@SuppressWarnings("unused")
	@Test
	public void testZeichenketteStringString() {
		Assert.assertEquals("", new Zeichenkette("Heinz", "").get());
		Assert.assertEquals(null, new Zeichenkette("Heinz", null).get());
		Assert.assertEquals("Willi", new Zeichenkette("Heinz", "Willi").get());

		thrown.expect(IllegalArgumentException.class);
		new Zeichenkette(null, "Willi");
	}

}
