/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.lib.daten;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

/**
 * Klasse zum Test der Klasse Zeitstempel.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ZeitstempelTest {

	/** Regel zur Auswertung erwarteter Exceptions. */
	@Rule
	public ExpectedException thrown = ExpectedException.none();

	/**
	 * Test: Attributname ermitteln.
	 */
	@Test
	public void testGetName() {
		final Zeitstempel zeit = new Zeitstempel("Heinz");
		Assert.assertEquals("Heinz", zeit.getName());
	}

	/**
	 * Test: Serialisierung und Deserialisierung.
	 */
	@Test
	public void testSerialization() {
		Zeitstempel zeit = new Zeitstempel("ABC", 12L);
		Assert.assertEquals("ABC", zeit.getName());

		try {
			final ByteArrayOutputStream byteOut = new ByteArrayOutputStream(1000);
			final ObjectOutputStream output = new ObjectOutputStream(byteOut);
			output.writeObject(zeit);
			final byte[] bytes = byteOut.toByteArray();

			final ObjectInputStream input = new ObjectInputStream(new ByteArrayInputStream(bytes));
			zeit = (Zeitstempel) input.readObject();
			Assert.assertEquals((Long) 12L, zeit.get());
		} catch (final ClassNotFoundException | IOException e) {
			Assert.fail(e.getLocalizedMessage());
		}
	}

	/**
	 * Test: Wert setzen.
	 */
	@Test
	public void testSet() {
		final Zeitstempel zeit = new Zeitstempel("Heinz");
		zeit.set(10L);
		Assert.assertEquals((Long) 10L, zeit.get());

		thrown.expect(IllegalArgumentException.class);
		zeit.set(null);
	}

	/**
	 * Test: Konstruktor mit Attributname.
	 */
	@SuppressWarnings("unused")
	@Test
	public void testZeitstempelString() {
		Assert.assertEquals((Long) 0L, new Zeitstempel("Heinz").get());

		thrown.expect(IllegalArgumentException.class);
		new Zeitstempel(null);
	}

	/**
	 * Test: Konstruktor mit Attributname und Wert.
	 */
	@SuppressWarnings("unused")
	@Test
	public void testZeitstempelStringLong() {
		Assert.assertEquals((Long) 0L, new Zeitstempel("Heinz", 0L).get());
		Assert.assertEquals((Long) (-1 * 10L), new Zeitstempel("Heinz", -10L).get());
		Assert.assertEquals((Long) 10L, new Zeitstempel("Heinz", 10L).get());
		Assert.assertEquals((Long) Long.MAX_VALUE, new Zeitstempel("Heinz", Long.MAX_VALUE).get());
		Assert.assertEquals((Long) Long.MIN_VALUE, new Zeitstempel("Heinz", Long.MIN_VALUE).get());

		thrown.expect(IllegalArgumentException.class);
		new Zeitstempel(null, 27L);
	}
}
