/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.param;

import java.util.Set;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.param.hierarchie.ParametrierungsHierarchieManager;
import de.bsvrz.puk.param.lib.Parameter;
import de.bsvrz.puk.param.lib.ParameterInfo;

/**
 * Schnittstelle zur Realisierung des Parametrierungsmoduls.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public interface ParametrierungsModul {

	/**
	 * führt alle Operationen zum Beenden des Moduls aus.
	 */
	void close();

	/**
	 * ermittelt den Parameterdatensatz für die übergebene
	 * Parameterspezifikation.
	 *
	 * @param info
	 *            die Spezifikation
	 * @param datenLiefern
	 *            gibt an, ob im Ergebnis die Daten selbst mit übertragen werden
	 *            sollen
	 * @return den Parameter-Datensatz
	 */
	Parameter ermittleParameter(ParameterInfo info, boolean datenLiefern);

	/**
	 * liefert den Manager zur Verwaltung der Objekthierarchie.
	 *
	 * @return den Manager
	 */
	ParametrierungsHierarchieManager getHierarchieManager();

	/**
	 * liefert das Systemobjekt, mit dem die Parametrierung in der
	 * Datenverteilerkonfiguration repräsentiert ist.
	 *
	 * @return das Objekt
	 */
	SystemObject getParameterObjekt();

	/**
	 * liefert das Persistenzmodul der Parametrierung.
	 *
	 * @return das Modul
	 */
	Persistenz getPersistenz();

	/**
	 * liefert den verwendeten Parameterprovider.
	 *
	 * @return den Provider
	 */
	Provider getProvider();

	/**
	 * liefert die verwendete Datenverteilerverbindung.
	 *
	 * @return die Verbindung
	 */
	ClientDavInterface getVerbindung();

	/**
	 * initialisiert die Daten für die übergebene Simulationsvariante.
	 *
	 * @param simulationsVariante
	 *            die Simulationsvariante
	 * @param parameter
	 *            die Parameterspezifikationen, für die die Simulation
	 *            parametriert ist.
	 */
	void initialisiereSimulationsParameter(short simulationsVariante, Set<ParameterInfo> parameter);
}
