/*
 * Segment 8 Parametrierung und Konfiguration, SWE 8.2 Parametrierung
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.puk.param.param;

import java.util.Collection;
import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.puk.param.lib.MethodenBibliothek;
import de.bsvrz.puk.param.lib.Parameter;
import de.bsvrz.puk.param.lib.ParameterInfo;
import de.bsvrz.puk.param.lib.daten.DataWithTime;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;

/**
 * Beispielanwendung zum Test der Blockierung des Datenverteilers beim Zugriff
 * auf die Konfiguration innerhalb einer Datenverteiler-Callback-Funktion.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class TestParametrierung implements StandardApplication, ClientSenderInterface {

	/**
	 * Hauptfunktion der Anwendung.
	 *
	 * @param args
	 *            die übergebenen Argumente
	 */
	public static void main(final String[] args) {
		StandardApplicationRunner.run(new TestParametrierung(), args);
	}

	/**
	 * der verwendete Aspekt.
	 */
	private Aspect defaultAspect;

	/**
	 * die verwendete Datenverteilerverbindung.
	 */
	private ClientDavInterface verbindung;

	@Override
	public void dataRequest(final SystemObject object, final DataDescription dataDescription, final byte state) {
		System.err.println("Request: " + object + ": " + dataDescription + " STATE: " + state);

		if (state == ClientSenderInterface.START_SENDING) {

			System.err.println("Versuche die Standardparameter zu ermitteln und zu senden ....");

			// Versuch 1:

			// System.err.println("Versuche die Standardparameter zu ermitteln
			// und zu senden ....");
			// Data data =
			// object.getConfigurationData(dataDescription.getAttributeGroup(),
			// defaultAspect);
			// System.err.println("Die Standardparameter sind: " + data);

			// Versuch 2:

			try {
				final ClientDavInterface dav = new ClientDavConnection(verbindung.getClientDavParameters());
				dav.connect();
				dav.login();
				final DataModel model = dav.getDataModel();
				final SystemObject objectMitNeuerVerbindung = model.getObject(object.getId());
				final Data data = objectMitNeuerVerbindung.getConfigurationData(dataDescription.getAttributeGroup(),
						defaultAspect);
				System.err.println("Die Standardparameter sind: " + data);
			} catch (final MissingParameterException e) {
				e.printStackTrace();
			} catch (final InconsistentLoginException e) {
				e.printStackTrace();
			} catch (final CommunicationError e) {
				e.printStackTrace();
			} catch (final ConnectionException e) {
				e.printStackTrace();
			}
		}
	}

	@Override
	public void initialize(final ClientDavInterface connection) throws Exception {

		this.verbindung = connection;

		final DataModel model = connection.getDataModel();
		//
		final SystemObject object = model.getObject("fs.mq.a10.0000.1üfs");
		final AttributeGroup atg = model.getAttributeGroup("atg.verkehrsDatenKurzZeitIntervallMessWertErsetzung");

		final SystemObject result = MethodenBibliothek.getHierarchie(connection)
				.getParameterWurzel(new ParameterInfo(object, atg));
		System.err.println("Wurzel: " + result);

		final List<AttributeGroup> atgs = object.getType().getAttributeGroups();
		final ParameterInfo[] infos = new ParameterInfo[atgs.size()];
		int idx = 0;
		for (final AttributeGroup atgElement : atgs) {
			infos[idx++] = new ParameterInfo(object, atgElement);
		}

		final Parameter[] parameterListe = MethodenBibliothek.getParameterManager(connection).getParameter(infos);
		for (final Parameter parameter : parameterListe) {
			System.err.println("Parameter: " + parameter);
		}

		final SystemObject mq = connection.getDataModel().getObject("mq.a10.0000");
		Collection<SystemObject> konfigurierbarListe = MethodenBibliothek.getHierarchie(connection)
				.getAenderbareObjekte(new ParameterInfo(mq, object.getType(), atg));
		for (final SystemObject konfigurierbar : konfigurierbarListe) {
			System.err.println("Konfigurierbar (mq.a10.0000): " + konfigurierbar);
		}

		connection.sleep(5000L);

		final Data data = connection.createData(atg);
		data.getTimeValue("MaxErsetzungsDauer").setSeconds(88);
		data.getTimeValue("MaxWiederholungsZeit").setSeconds(4444);
		data.getUnscaledValue("MaxWiederholungAnzahl").set(888.66f);

		// MethodenBibliothek.getParameterManager(connection).setParameter(new
		// Parameter(new ParameterInfo(object, atg)));
		final Parameter parameter = new Parameter(new ParameterInfo(mq, object.getType(), atg));
		parameter.setDataWithTime(new DataWithTime(data, System.currentTimeMillis()));
		// MethodenBibliothek.getParameterManager(connection).setParameter(parameter);
		MethodenBibliothek.getParameterManager(connection).setParameter(parameter);
		// MethodenBibliothek.getParameterManager(connection).setParameter(new
		// Parameter(new ParameterInfo(mq.getType(), object.getType(), atg)));

		konfigurierbarListe = MethodenBibliothek.getHierarchie(connection)
				.getAenderbareObjekte(new ParameterInfo(object, object.getType(), atg));
		for (final SystemObject konfigurierbar : konfigurierbarListe) {
			System.err.println("Konfigurierbar (fs.mq.a10.0000.1üfs): " + konfigurierbar);
		}

		// atg = model
		// .getAttributeGroup("atg.verkehrsDatenDifferenzialKontrolleFs");
		// Parameter[] parameterListe = MethodenBibliothek
		// .getParameterManager(connection).getParameter(
		// new ParameterInfo(object, atg));

		final Collection<SystemObject> objects = model.getObjects("simulation.001", 0L, System.currentTimeMillis());
		connection.getLocalConfigurationAuthority().getMutableSet("Simulationen")
				.add(objects.toArray(new SystemObject[0])[0]);

		// SystemObject obj = model.getObject("simulation.001");
		// connection.getLocalConfigurationAuthority().getMutableSet("Simulationen").add(obj);

		// SystemObject obj = model.getObject("simulationsstrecke.1");
		// connection.getLocalConfigurationAuthority().getMutableSet("SimulationsStrecken").add(obj);
	}

	@Override
	public boolean isRequestSupported(final SystemObject object, final DataDescription dataDescription) {
		return true;
	}

	@Override
	public void parseArguments(final ArgumentList argumentList) throws Exception {
		// keine Parameter auszuwerten

	}
}
